import pytest
from journeylib.funciones import ins_historico
from fastapi.responses import JSONResponse
import json
    
def test_ins_historico_usr_vacio():
    usr = ''
    num_chat = 1
    contexto = 'contexto'
    lista_msg = [{"role": "user", "content": "Hola, ¿cómo estás?"},
                 {"role": "assistant", "content": "Hola, soy ia."}]
    tipoIA = 'PASADO'
    cerrado = True
    url='http://ismi.fi.upm.es:8080/insertar/chat'

    status_code = ins_historico(usr, num_chat, contexto, lista_msg, tipoIA, cerrado, url)
    assert status_code != 200

def test_ins_historico_lista_msg_vacia():
    usr = 'usuario'
    num_chat = 1
    contexto = 'contexto'
    lista_msg = []
    tipoIA = 'PASADO'
    cerrado = True
    url='http://ismi.fi.upm.es:8080/insertar/chat'

    status_code = ins_historico(usr, num_chat, contexto, lista_msg, tipoIA, cerrado, url)
    assert status_code != 200

def test_ins_historico_tipoIA_invalido():
    usr = 'usuario'
    num_chat = 1
    contexto = 'contexto'
    lista_msg = []
    tipoIA = 'PASADO'
    cerrado = True
    url='http://ismi.fi.upm.es:8080/insertar/chat'

    tipos_validos = ['PASADO', 'PRESENTE', 'FUTURO', 'COMPRA']
    if tipoIA not in tipos_validos:
        raise ValueError("El valor de tipoIA debe ser uno de {tipos_validos}")
    
    with pytest.raises(ValueError):
        ins_historico(usr, num_chat, contexto, lista_msg, tipoIA, cerrado, url)

def test_ins_historico_url_invalido():
    usr = 'usuario'
    num_chat = 1
    contexto = 'contexto'
    lista_msg = []
    tipoIA = 'PASADO'
    cerrado = True
    url='aaa'

    url_valida = ['http://ismi.fi.upm.es:8080/insertar/chat']
    if url not in url_valida:
        raise ValueError("Url invalida")
    
    with pytest.raises(ValueError):
        ins_historico(usr, num_chat, contexto, lista_msg, tipoIA, cerrado, url)
 
"""
def test_ins_historico():
    usr = 'usuario'
    num_chat = 1
    contexto = 'contexto'
    lista_msg = [{"role": "user", "content": "Hola, ¿cómo estás?"},
                 {"role": "assistant", "content": "Hola, soy ia."}]
    tipoIA = 'PASADO'
    cerrado = True
    datos=[usr, num_chat, contexto, lista_msg, tipoIA, cerrado]

    response = ins_historico(usr, num_chat, contexto, lista_msg, tipoIA, cerrado)
    assert response.status_code == 200

    expected_data = {
        'id_usuario': usr,
        'num_chat': num_chat,
        'contexto': contexto,
        'lista_msg': lista_msg,
        'tipoIA': tipoIA,
        'cerrado': cerrado
    }
    # Convierte la respuesta JSON a un diccionario para comparar fácilmente
    response_data = json.loads(response.content)
    assert response_data == expected_data
"""


import pytest
import logging
import journeylib as jl

# res = jl.ins_historico('Nelson', 'PRESENTE', 1, lista_msga, 'holita', url='http://localhost:5000/insertar/chat')
# res = jl.get_historico('Nelson', 'PRESENTE', 1, url='http://localhost:5000/read/chats')

def test_get_historico_exitoso():
    res = jl.get_historico('Nelson', 'PRESENTE', 1, url='http://localhost:5000/read/chats')
    assert res[0]['role'] == 'system'
    assert res[1]['role'] == 'user'
    assert res[2]['role'] == 'assistant'
    print('\n\ntest_get_historico_exitoso, lista de mensajes devueltos:')
    for msg in res:
        print(msg)

def test_ins_historico_exitoso():
    lista_msgs = [ {'role': 'user', 'content': 'Por favor funciona'}, {'role': 'assistant', 'content': 'Ok ahora quiero funcionar'}]
    res = jl.ins_historico('Nelson', 'PRESENTE', 1, lista_msgs, 'holita', url='http://localhost:5000/insertar/chat')
    assert type(res) is int
    assert res == 1



#Imports necesarios para las funciones de la libreria
import requests

#Metodo de prueba para el despliege de la libreria
def hola_journey():
    print('Holitaa, esta es una librería diseñada y publicada por JourneyGen para el proyecto GPTravel.')

"""
Endpoint para insertar:
http://ismi.fi.upm.es:8080/insertar/chat
body{

}
response{

}

Endpoint para recibir:
servidor/read/chats
body{
    username: (nombre de usuario),
    chat: (int, numero de chat),
    tipoIA: (enum, PASADO PRESENTE FUTURO COMPRA)
}
response{
    mensajes: [
        {
            fecha: (formato SQL),
            input: (content del usuario),
            output: (content de la ia)
        }
    ]
}
"""

#Implementacion de las funciones necesarias para la libreria
"""
METODO INSERTAR HISTORICO
Descripcion: Inserta un nuevo par de mensajes a la bd de CASH
"""
def ins_historico(usr, tipoIA, num_chat, lista_msg, contexto, cerrado=False, url='http://195.35.1.47:8080/insertar/chat'):
    """
    PRE:
        usr es un nombre de usuario valido
        tipoIA es un string de entre la lista ['PASADO', 'PRESENTE', 'FUTURO', 'COMPRA']
        num_chat es un int coherente con una de las siguientes descripciones:
            1. identificador de chat existente
            2. None (en este caso crearía un chat nuevo)
        lista_msg es una lista de diccionarios formato {'role':'', 'content':''}
        contexto es un string con el contexto que use la ia que esté teniendo la conversación
    POST:
        inserta un nuevo mensaje en la BD de CASH
    DEVUELVE:
        (int): valor identificativo asociado al chat sobre el que se insertan los mensajes
    """
    try:
        datos_post = { # Formato con el que se consume la api
            'usr': usr,
            'num_chat': num_chat,
            'contexto': contexto,
            'lista_msg': lista_msg,
            'tipoIA': tipoIA,
            'cerrado': cerrado
        }
        response = requests.post(url, json=datos_post)

        if response.status_code == 200:
            n_chat = response.json()['num_chat'] # Obtenemos el numero de chat devuelto al hacer la solicitud
            return n_chat
        else:
            raise ValueError(f"ins_historico, codigo de respuesta {response.status_code}, detail: {response.text}") # Codigo de respuesta distinto de 200 (no exitoso)
    except Exception as e: # Excepcion ocurrida al hacer la peticion
        raise e

def get_historico(usr, tipoIA, num_chat, url='http://195.35.1.47:8080/read/chats'):
    """
    PRE:
        usr es un nombre de usuario valido
        tipoIA es un string de entre la lista ['PASADO', 'PRESENTE', 'FUTURO', 'COMPRA']
        num_chat es un int correspondiente a un identificador de chat existente
    DEVUELVE:
        una lista de diccionarios (json) con el siguiente formato:
        {'role':'', 'content':''}
        El primer mensaje será con role='system' y content='contexto de la ia'
        El resto de los mensajes serán pares que alternarán entre mensajes del usuario y la ia
    """
    msgs = [] # Lista donde se guardaran todos los mensajes que se pasaran a la IA por el create
    
    try:
        datos_post = { # Formato con el que se consume la api
            'username': usr,
            'chat': num_chat,
            'tipoIA': tipoIA
        }
        
        response = requests.post(url, json=datos_post)

        if response.status_code == 200:
            """
             Obtenemos cuerpo, lista de dict tq: {
                'fecha': mensaje.fecha.strftime("%Y-%m-%d %H:%M"),
                'input': mensaje.input,
                'output': mensaje.output,
                'cerrado': mensaje.cerrado,
                'contexto': mensaje.contextoia
            }
            """
            data = response.json()
            
            msg_contexto = {'role': 'system', 'content': data[0]['contexto']}
            msgs.append(msg_contexto)
            for msg in data:
                msg_usuario = {'role': 'user', 'content': msg['input']}
                msg_ia = {'role': 'assistant', 'content': msg['output']}
                msgs.append(msg_usuario)
                msgs.append(msg_ia)
            
            return msgs

        else:
            raise ValueError(f"ins_historico, codigo de respuesta {response.status_code}") # Codigo de respuesta distinto de 200 (no exitoso)
    except Exception as e: # Excepcion ocurrida al hacer la peticion
        raise e

def get_historico_falso(usr, url='http://farlier.org:8080/chat/obtener'):

    datos_post = {
        'usr': usr
    }
    response = requests.post(url, json=datos_post)

    return {'respuesta': response.text, 'codigo': response.status_code}

def ins_historico_falso(usr, msgs, url='http://farlier.org:8080/chat/insertar'):

    datos_post = {
        'usr': usr,
        'msgs': msgs
    }
    response = requests.post(url, json=datos_post)

    return {'respuesta': response.text, 'codigo': response.status_code}


import pathlib
from setuptools import find_packages, setup

HERE = pathlib.Path(__file__).parent

VERSION = '2.0.12' #Numero de version que decidamos, cambiarlo al añadir nuevas funcionalidades o cambios
PACKAGE_NAME = 'journeylib' #Nombre de la libreria
AUTHOR = 'Equipo JourneyGen UPM' #Modificar con vuestros datos
AUTHOR_EMAIL = 'f.gonzalez.lopez@alumnos.upm.es' #Modificar con vuestros datos
URL = 'https://ismigit.fi.upm.es/gptravel-2024/ai-squad/journeygenai' #Modificar con vuestros datos

LICENSE = 'MIT' #Tipo de licencia
DESCRIPTION = 'Libreria para gestionar datos del historico del proyecto GPTravel' #Descripción corta
LONG_DESCRIPTION = (HERE / "README.md").read_text(encoding='utf-8') #Referencia al documento README con una descripción más elaborada
LONG_DESC_TYPE = "text/markdown"


#Paquetes necesarios para que funcione la libreía. Se instalarán a la vez si no lo tuvieras ya instalado
INSTALL_REQUIRES = [
    'fastapi',
    'requests'
      ]

setup(
    name=PACKAGE_NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type=LONG_DESC_TYPE,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    url=URL,
    install_requires=INSTALL_REQUIRES,
    license=LICENSE,
    packages=find_packages(),
    include_package_data=True
)